package DC;

import java.util.*;
import javax.swing.JOptionPane;
import java.awt.Color;

import cytoscape.CyNetwork;
import cytoscape.CyNode;
import cytoscape.Cytoscape;
import cytoscape.giny.CytoscapeRootGraph;
import cytoscape.view.CyNetworkView;
import giny.model.Node;
import giny.view.EdgeView;
import giny.view.NodeView;

public class CriticalPath{
    private CyNetwork network;
    private CyNetworkView view;
    private CytoscapeRootGraph root;
    private boolean undirected;

    public static final int WHITE = 0, GRAY = 1, BLACK = 2;
    private int[][] capacity;
    private int[] parent, color, queue;
    private int size, source, sink;
    private int[] indexArray, adjArray;
    private boolean[] visited;
    private Boolean[] inAllPaths;
    private Stack<Integer> path;
    private boolean[][] pathMatrix;
    private int run = 0;
    public static ArrayList<Boolean[]> pathMatrix2 = new ArrayList<Boolean[]>();
    public String CPSaveData;

    // Get the selected nodes and calculate the critical path
	public void result(boolean directed) {
            network = Cytoscape.getCurrentNetwork();
            view = Cytoscape.getCurrentNetworkView();
            root = Cytoscape.getRootGraph();
            undirected = true;

            int[] selNodes = view.getSelectedNodeIndices();

            if (selNodes.length != 2) {
                JOptionPane.showMessageDialog(view.getComponent(),
                "You must first select 2 nodes (use shift) in order to get the max flow between them.");
            }
            int node1 = selNodes[1];
            int node2 = selNodes[0];

            boolean[][] allPaths = getAllPaths(node1,node2);

            if (allPaths == null){
                JOptionPane.showMessageDialog(view.getComponent(),
                "There is no path between the selected nodes");
		}
                 else {}
	}

        public boolean[][] getAllPaths(int node1, int node2) {

            indexArray = root.getNodeIndicesArray();
            size = indexArray.length;
            String output = "";
            int colorIndex = 0;
            int indexCrit = 0;
            int critLength = 0;
            CPSaveData = "";

            parent = new int[size];
            color = new int[size];
            queue = new int[size];
            capacity = new int[size][size];
            int index = 0;
            inAllPaths = new Boolean[size];
            java.util.Arrays.fill(inAllPaths,true); // init to all true
            visited = new boolean[size];
            java.util.Arrays.fill(visited,false); // init to all false
            path = new Stack<Integer>();
            path.clear(); // init empty
            pathMatrix = new boolean[size][size];

            CyNode cynode1 = (CyNode) network.getNode(node1);
            CyNode cynode2 = (CyNode) network.getNode(node2);
            source = -1*root.getIndex(cynode2)-1;
            sink = -1*root.getIndex(cynode1)-1;

            //Build capacity[size][size] matrix
            for (int i = 0; i < size; i++){
             adjArray = network.neighborsArray(-i-1);
             for (int j = 0; j < adjArray.length; j++){
                            index = -1*adjArray[j]-1;
                                 capacity[i][index] = 1 ;//1 means connected
                }
            }
            Arrays.fill(inAllPaths,true); // init to all true
            Arrays.fill(visited,false); // init to all false
            path.clear(); // init empty

            DFS(source);

            int tempCrit = 0;
            int maxCrit = 0;
            int row = 0;
            int tempLength = 0;
            //Calculate critical path from all paths
            for(int i=0; i<pathMatrix2.size(); i++){
                Boolean[] temp = pathMatrix2.get(i);
                tempCrit = 0;
                for(int j=0; j<temp.length; j++){
                    if(temp[j] == true){
                        indexCrit = -1*j - 1;
                        tempCrit += network.getDegree(indexCrit);
                        tempLength += 1;
                        if(tempCrit > maxCrit){
                            maxCrit = tempCrit;
                            critLength = tempLength-1;
                            row = i;
                        }
                    }else{}
                }
            }
            //Color nodes in critical path
            Boolean[] temp = pathMatrix2.get(row);
            ArrayList<String> idArray = new ArrayList<String>();
            String NodeID ="";
            for(int k=0; k<temp.length; k++){
                    if(temp[k] == true){
                        colorIndex = -1*k - 1; // Makes index negative and adjusted
                        Node node = root.getNode(colorIndex); //Get node by index
                        NodeView nodeView = view.getNodeView(node);
                        nodeView.setUnselectedPaint(Color.BLUE);
                        //Fill idArray
                        Node node3 = root.getNode(colorIndex);
                        NodeID = node3.getIdentifier();
                        idArray.add(NodeID);
                    }else{}
                }
            //Send idArray to string
            for(int k=0; k<idArray.size(); k++){
                output += " " + idArray.get(k);
            }

            //Change color of source and sink nodes
            NodeView nodeView1 = view.getNodeView(cynode1); //Source = "Go"
            nodeView1.setUnselectedPaint(Color.GREEN);
            String sourceNodeID = cynode1.getIdentifier();
            NodeView nodeView2 = view.getNodeView(cynode2); //Sink = "Stop"
            nodeView2.setUnselectedPaint(Color.RED);
            String sinkNodeID = cynode2.getIdentifier();

            JOptionPane.showMessageDialog(Cytoscape.getDesktop(), "Critical Path Value: " + maxCrit + "\n\nCritical Path Length: " + critLength);
            pathMatrix2.clear();

            //Save Data
            CPSaveData = "Critical Path("+sourceNodeID+", "+sinkNodeID+"): "
                    +"["+ output +"]"+ "   Length: " +critLength;
            SaveData SDobj = new SaveData();
            SDobj.WriteFile(CPSaveData);

            return pathMatrix;
        }

        //Depth First Search
        private boolean DFS(int u){
            if (visited[u] == true){
                return true;
            }
                if (u == sink){
                    for (int i = 0; i < size; i++){
                         inAllPaths[i] = true;
                    }
                    for (int i = 0; i < size; i++){
                        if (!path.contains(i)){
                            inAllPaths[i] = false; // inAllPaths contains true only at vertices that exist in all paths from source to sink.
                            }
                    }
                    inAllPaths[sink] = true;
                    pathMatrix2.add(deepCopy(inAllPaths));
                    return true;
                 }else {
                    visited[u] = true;
                    path.push(u);
                    //foreach edge (u, v)
                    for (int k = 0; k < size; k++){
                        int v = capacity[u][k];
                        if (v != 0 && visited[k] != true){
                            DFS(k);
                        }
                    }
                    path.pop();
                    visited[u] = false;
                    return false;
                                }
                            }
        static Boolean[] deepCopy(Boolean[] orig) {
        Boolean[] copy = new Boolean[orig.length];
            copy = orig.clone();
        return copy;
    }
}
