package DC;

import javax.swing.JOptionPane;
import java.awt.Color;

import cytoscape.CyNetwork;
import cytoscape.CyNode;
import cytoscape.Cytoscape;
import cytoscape.giny.CytoscapeRootGraph;
import cytoscape.view.CyNetworkView;
import giny.model.Node;
import giny.view.EdgeView;
import giny.view.NodeView;
import java.util.ArrayList;

public class MaxFlow {
    private CyNetwork network;
    private CyNetworkView view;
    private CytoscapeRootGraph root;
    private EdgeView edgeView;
    private boolean undirected;

    public static final int WHITE = 0, GRAY = 1, BLACK = 2;
    private int[][] flow, capacity, res_capacity;
    private int[] parent, color, queue;
    private int[] min_capacity;
    private int size, source, sink, first, last;
    private int max_flow;
    private int[] indexArray, adjArray;
    public String MFSaveData;

    // Get the selected nodes and calculate the max flow
	public void result(boolean directed) {
            network = Cytoscape.getCurrentNetwork();
            view = Cytoscape.getCurrentNetworkView();
            root = Cytoscape.getRootGraph();
            undirected = true;

            int[] selNodes = view.getSelectedNodeIndices();

            if (selNodes.length != 2) {
                JOptionPane.showMessageDialog(view.getComponent(),
                "You must first select 2 nodes (use shift) in order to get the max flow between them.");
            }
            int node1 = selNodes[1];
            int node2 = selNodes[0];

            int[][] maxFlow = getMaxFlow(node1,node2);

            if (maxFlow == null){
                JOptionPane.showMessageDialog(view.getComponent(),
                "There is no path between the selected nodes");
		}
                 else {}
	}
        // Edmonds-Karp algorithm
        public int[][] getMaxFlow(int node1, int node2) {

            indexArray = root.getNodeIndicesArray();
            size = indexArray.length;
            String output = "";

            flow = new int[size][size];
            res_capacity = new int[size][size];
            parent = new int[size];
            min_capacity = new int[size];
            color = new int[size];
            queue = new int[size];
            capacity = new int[size][size];
            int index = 0;
            int colorIndex = 0;
            MFSaveData = "";

            CyNode cynode1 = (CyNode) network.getNode(node1);
            CyNode cynode2 = (CyNode) network.getNode(node2);
            source = -1*root.getIndex(cynode2)-1;
            sink = -1*root.getIndex(cynode1)-1;

            //Build capacity[size][size] matrix
            for (int i = 0; i < size; i++){
             adjArray = network.neighborsArray(-i-1);
             for (int j = 0; j < adjArray.length; j++){
                            index = -1*adjArray[j]-1;
                            //nodeIndex = -1*adjArray[j];
                                 capacity[i][index] = 1 ;//1 means connected
                }
            }

                for (int i = 0; i < size; i++)
                        for (int j = 0; j < size; j++)
                                res_capacity[i][j] = capacity[i][j];

                while (BFS(source)){
                        max_flow += min_capacity[sink];
                        int v = sink, u;
                        while (v != source){
                                u = parent[v];
                                flow[u][v] += min_capacity[sink];
                                flow[v][u] -= min_capacity[sink];
                                res_capacity[u][v] -= min_capacity[sink];
                                res_capacity[v][u] += min_capacity[sink];
                                v = u;
                        }
                }

            //Change Node Color & Print matrix for debugging
            ArrayList<String> idMatrix = new ArrayList<String>();
            String NodeID ="";
            for (int row = 0; row < size; row++) {
            for (int col = 0; col < size; col++) {
                if(flow[row][col] != 0){
                    colorIndex = -1*row - 1; // Makes certain index is negative and adjusted
                    Node node = root.getNode(colorIndex); //Get node by index
                    NodeView nodeView = view.getNodeView(node);
                    nodeView.setUnselectedPaint(Color.BLUE);
                    //Fill idArray
                    Node node3 = root.getNode(colorIndex);
                    NodeID = node3.getIdentifier();
                    idMatrix.add(NodeID);
                }else{}
            }
        }
            //Send idArray to string
            for(int k=0; k<idMatrix.size(); k++){
                output += " " + idMatrix.get(k);
            }

            //Change color of source and sink nodes
            NodeView nodeView1 = view.getNodeView(cynode1); //Source = "Go"
            nodeView1.setUnselectedPaint(Color.GREEN);
            String sourceNodeID = cynode1.getIdentifier();
            NodeView nodeView2 = view.getNodeView(cynode2); //Sink = "Stop"
            nodeView2.setUnselectedPaint(Color.RED);
            String sinkNodeID = cynode2.getIdentifier();

            JOptionPane.showMessageDialog(Cytoscape.getDesktop(), "Max Flow: " + max_flow);

            //Save Data
            MFSaveData = "Max Flow Path("+sourceNodeID+", "+sinkNodeID+"): "
                    +"["+ output +"]"+ "   Flow Value: " +max_flow;
            SaveData SDobj = new SaveData();
            SDobj.WriteFile(MFSaveData);

            return flow;
        }

        //Breadth First Search
        private boolean BFS(int source){
                for (int i = 0; i < size; i++){
                        color[i] = WHITE;
                        min_capacity[i] = Integer.MAX_VALUE;
                }

                first = last = 0;
                queue[last++] = source;
                color[source] = GRAY;

                // While "queue" not empty
                while (first != last){
                        int v = queue[first++];
                        for (int u = 0; u < size; u++)
                                if (color[u] == WHITE && res_capacity[v][u] > 0){
                                        min_capacity[u] = Math.min(min_capacity[v], res_capacity[v][u]);
                                        parent[u] = v;
                                        color[u] = GRAY;
                                        if (u == sink) return true;
                                        queue[last++] = u;
                                }
                }
                return false;
        }

}
